#!/usr/bin/env python2.5

import depsd

from meresco.framework import Transparant, Observable, be

from meresco.components import readConfig, StorageComponent, XmlParseAmara, XmlPrintLxml, Amara2Lxml, RewritePartname, Lxml2Amara
from meresco.components.http import ObservableHttpServer, PathFilter
from meresco.components.oai import UniqueNumberGenerator, OaiJazzLucene, OaiMain, OaiIdentify, OaiGetRecord, OaiList, OaiListMetadataFormats, OaiListSets, OaiSink
from meresco.components.lucene import unlock, LuceneIndex
from meresco.components.http.webrequestserver import WebRequestServer

from weightless import Reactor, compose
from cq2utils import DirectoryWatcher

from os.path import dirname, join, isdir
from os import rename, makedirs
from time import strftime, localtime
from sys import stdout, argv

from glob import glob

from bzv.utils import HandleRequestPrinter, IdentifierPrinter, Printer
from bzv import EditorialItemProcessor, Mods2Dc
from bzv.elsevierreader import readToc

class PartnameFilter(Transparant):
    def __init__(self, partnames):
        Transparant.__init__(self)
        self._partnames = partnames

    def add(self, id, partname, data):
        if partname in self._partnames:
            yield self.all.add(id, partname, data)

class swallowStopIteration:

    def __init__(self, f):
        self._f = f

    def __call__(self):
        try:
            self._f()
        except StopIteration:
            pass

class ProcessElsevierData(object):

    def __init__(self, reactor, event, inboxDirectory, doneDirectory, editorialItemProcessor):
        self._reactor = reactor
        self._donePath = join(doneDirectory, event.name)
        self._tocDirname = join(inboxDirectory, event.name)
        self._tocFile = open(join(inboxDirectory, event.name, 'dataset.toc'))

        self._editorialItemProcessor = editorialItemProcessor

        generator = self.do()
        self._reactor.addReader(self._tocFile, swallowStopIteration(generator.next))

    def do(self):
        try:
            f = self._editorialItemProcessor.process(self._tocDirname)
            f.next()
            generator = compose(readToc(f))
            generator.next()

            yield
            line = self._tocFile.readline()
            while line:
                generator.send(line)
                yield
                line = self._tocFile.readline()
            generator.close()

            rename(self._tocDirname, self._donePath + strftime('_%Y%m%d-%H%M%S', localtime()))
        finally:
            self._reactor.removeReader(self._tocFile)



def main(reactor, observableHttpServer, inboxDirectory, doneDirectory, databasePath, oaiAdminEmail, oaiRepositoryName):

    oaiMetaIndexDirectory = join(databasePath, 'oai', 'metaIndex')
    oaiStorageDirectory = join(databasePath, 'oai', 'storage')
    uniquenumberFilename = join(databasePath, 'oai', 'unique')

    oaiStorageComponent = StorageComponent(oaiStorageDirectory)

    editorialItemProcessor = EditorialItemProcessor()

    unlock(oaiMetaIndexDirectory)
    oaiMetaIndex = LuceneIndex(oaiMetaIndexDirectory, reactor, bitwise=False)
    numberGenerator = UniqueNumberGenerator(uniquenumberFilename)
    oaiJazzComponent = OaiJazzLucene(oaiMetaIndex, oaiStorageComponent, numberGenerator)

    if glob(inboxDirectory + "/*"):
        raise Exception('inboxDirectory %s not empty: %s' % (inboxDirectory, glob(inboxDirectory + "/*")))

    isdir(inboxDirectory) or makedirs(inboxDirectory)
    isdir(doneDirectory) or makedirs(doneDirectory)

    directoryWatcher = DirectoryWatcher(
        inboxDirectory,
        lambda event: ProcessElsevierData(reactor, event, inboxDirectory, doneDirectory, editorialItemProcessor),
        CreateFile=True, ModifyFile=True, MoveInFile=True)
    reactor.addReader(directoryWatcher, directoryWatcher)

    processor = \
    (Observable(),
        (editorialItemProcessor,
            (oaiStorageComponent,),
            (XmlParseAmara(),
                (PartnameFilter(['mods']),
                    (IdentifierPrinter('"Elsevier OAI input"'),),
                    (oaiJazzComponent,),
                    (Amara2Lxml(),
                        (Mods2Dc(),
                            (RewritePartname('oai_dc'),
                                (Lxml2Amara(), 
                                    (oaiJazzComponent,),
                                ),
                                (XmlPrintLxml(),
                                    (oaiStorageComponent,),
                                )
                            )
                        )
                    )
                ),
                (PartnameFilter(['header']),
                    (oaiJazzComponent,),
                )
            )
        )
    )
   
    server = \
    (Observable(),
        (observableHttpServer,
            (PathFilter('/elsevier/oai'),
                (HandleRequestPrinter(),),
                (WebRequestServer(), 
                    (OaiMain(), 
                        (OaiIdentify(repositoryName=oaiRepositoryName, adminEmail=oaiAdminEmail),),
                        (OaiGetRecord(), 
                            (oaiStorageComponent,),
                            (oaiJazzComponent,),
                        ),
                        (OaiList(), 
                            (oaiStorageComponent,),
                            (oaiJazzComponent,),
                        ),
                        (OaiListMetadataFormats(), 
                            (oaiJazzComponent,),
                        ),
                        (OaiListSets(), 
                            (oaiJazzComponent,),
                        ),
                        (OaiSink(),)
                    )
                )
            )
        )
    )
    return processor, server
